'use strict';

angular.module('services.common.api', ['ngResource', 'ionic', 'ionic.service.core', 'services.common.constants'])

// .factory('Network', ['$rootScope', 'Constants', '$cordovaNetwork', function(Constants, $rootScope, $cordovaNetwork) {
//   alert('great');
//   if(!$cordovaNetwork.isOnline()){
//     if(Constants.DEBUGMODE){
//       console.log("No NETWORK FOUND");
//       // at this point we need to broadcast a no network found event and the app.controller will get that and display the popup
//       $rootScope.$broadcast('event:app-networkRequired');
//     }
//   }else{
//     if(Constants.DEBUGMODE){
//       console.log("NETWORK FOUND");
//     }
//   }
// })

.factory('Topics', ['$resource', 'Constants', function($resource, Constants) {
  return $resource(Constants.API.baseUrl + '/topics.json');
}])

.factory('Topic', ['$resource', 'Constants', function($resource, Constants) {
  return $resource(Constants.API.baseUrl + '/topics/:id.json',
    {id:'@id'},
    {
      redeem: {
        method: 'POST',
        url: Constants.API.baseUrl + '/topics/:id/redeem.json'
      }
    });
}])

.factory('Posts', ['$resource', 'Constants', function($resource, Constants) {
  var actions = {
    'top100': {
      method: 'GET',
      isArray: true,
      params: {'list_type': 'top100'}
    },
    'latest': {
      method: 'GET',
      isArray: true,
      params: {'list_type': 'latest'}
    },
    'friends': {
      method: 'GET',
      isArray: true,
      params: {'list_type': 'friends'}
    },
    'random': {
      method: 'GET',
      isArray: true,
      params: {'list_type': 'random'}
    }
  };

  return $resource(Constants.API.baseUrl + '/topics/:topicId/posts.json', {topicId:'@topicId'}, actions);

}])

.factory('Post', ['$resource', 'Constants', function($resource, Constants) {
  return $resource(
    Constants.API.baseUrl + '/posts/:id.json',
    {topicId:'@topicId', id:'@id'},
    {
      save: {
        method: 'POST',
        url: Constants.API.baseUrl + '/topics/:topicId/posts'
      },
      abuse: {
        method: 'POST',
        url: Constants.API.baseUrl + '/posts/:id/abuses.json?type=post'
      }
    }
  );
}])

.factory('Comments', ['$resource', 'Constants', function($resource, Constants) {
  return $resource(Constants.API.baseUrl + '/posts/:postId/comments.json',  {postId:'@postId'});
}])

.factory('Comment', ['$resource', 'Constants', function($resource, Constants) {
  return $resource(
    Constants.API.baseUrl + '/comments/:id.json',
    {postId:'@postId', id:'@id'},
    {
      save: {
        method: 'POST',
        url: Constants.API.baseUrl + '/posts/:postId/comments'
      },
      delete: {
        method: 'DELETE',
        url: Constants.API.baseUrl + '/comments/:id.json'
      },
      abuse: {
        method: 'POST',
        url: Constants.API.baseUrl + '/comments/:id/abuses.json?type=comment'
      }
    }
  );
}])

.factory('Like', ['$resource', 'Constants', function($resource, Constants) {
  return $resource(
    Constants.API.baseUrl + '/likes/:id.json',
    {postId:'@postId', id:'@id'},
    {
      save: {
        method: 'POST',
        url: Constants.API.baseUrl + '/posts/:postId/likes'
      }
    }
  );
}])

.factory('Follow', ['$resource', 'Constants', function($resource, Constants) {
  return $resource(Constants.API.baseUrl + '/profiles/:profileId/follow.json', {profileId:'@profileId'});
}])

.factory('Profiles', ['$resource', 'Constants', function($resource, Constants) {
  return $resource(
    Constants.API.baseUrl + '/profiles.json',
    {text:'@text'},
    {
      defaults: {
        mehtod: 'GET',
        isArray: true,
        url: Constants.API.baseUrl + '/profiles/defaults'
      }
    }
  );
}])

.factory('Profile', ['$resource', 'Constants', function($resource, Constants) {
  return $resource(Constants.API.baseUrl + '/profiles/:id.json', {id:'@id'},
  {
    save: {
      method: 'PUT',
      url: Constants.API.baseUrl + '/profiles/:id.json'
    },
    updateActivitiesTimestamp: {
      method: 'PUT',
      url: Constants.API.baseUrl + '/profiles/update_timestamp.json?type=activities'
    },
    updateDiscoverTimestamp: {
      method: 'PUT',
      url: Constants.API.baseUrl + '/profiles/update_timestamp.json?type=discover'
    },
    changePassword: {
      method: 'PUT',
      url: Constants.API.baseUrl + '/users.json'
    },
    forgetPassword: {
      method: 'PUT',
      url: Constants.API.baseUrl + '/profiles/forget_password.json'
    },
    signOut: {
      method: 'DELETE',
      url: Constants.API.baseUrl + '/users/sign_out.json'
    },
    checkInvitationCode: {
      method: 'PUT',
      url: Constants.API.baseUrl + '/profiles/check_invitation_code.json'
    },
    followers: {
      mehtod: 'GET',
      isArray: true,
      url: Constants.API.baseUrl + '/profiles/:id/followers.json'
    },
    followings: {
      mehtod: 'GET',
      isArray: true,
      url: Constants.API.baseUrl + '/profiles/:id/followings.json'
    },
    abuse: {
      method: 'POST',
      url: Constants.API.baseUrl + '/profiles/:id/abuses.json?type=user'
    },
    block: {
      method: 'POST',
      url: Constants.API.baseUrl + '/profiles/:id/block.json'
    }
  });
}])

.factory('News', ['$resource', 'Constants', function($resource, Constants) {
  return $resource(Constants.API.baseUrl + '/news.json');
}])

.factory('Toasts', ['$resource', 'Constants', function($resource, Constants) {
  return $resource(Constants.API.baseUrl + '/toasts.json');
}])

.factory('Activities', ['$resource', 'Constants', function($resource, Constants) {
  return $resource(Constants.API.baseUrl + '/activities.json', {},
  {
    following: {
      method: 'GET',
      isArray: true,
      url: Constants.API.baseUrl + '/activities.json?type=following'
    },
    me: {
      method: 'GET',
      isArray: true,
      url: Constants.API.baseUrl + '/activities.json?type=me'
    }
  });
}])