'use strict';

/*
 * common/auth.js
 *
 * (c) 2014 Vincent Maliko http://frnchnrd.com
 * License: MIT
 */

angular.module('services.common.auth',['ionic', 'services.models.user'])

.service('AuthService', ['$http', '$rootScope', '$q', '$localstorage', '$translate', 'UserModel', 'Constants',
  function($http, $rootScope, $q, $localstorage, $translate, UserModel, Constants){

  var auth = {

    // the user currently logged in
    currentUser: {},

    init : function(){
      var self = this;

      //set the session token in the http header
      $http.defaults.headers.common['X-Access-Token'] = window.localStorage.getItem('sessionToken') || "";
      $http.defaults.headers.common['Accept'] = "application/json";
      $http.defaults.headers.post['Content-Type'] = "application/json";

      // getting the userdata if there's any from the localstorage
      var userData = window.localStorage.getItem('app_user') ||  {profileBg:"", avatar: "", objectId : "", username: "", name : "", bio:"",location:"",website:""};

      //quick check to make sure it's a string ()
      if(typeof(userData) === "string")
      {
        userData = JSON.parse(userData);
      }

      // setting currentUser
      self.currentUser = new UserModel(userData);
    },

    // ping the api/me url to get the currennt logged user
    ping: function(source){
      var self  = this;
      var deferred = $q.defer();

      var success = function(response, status, headers, config){
        deferred.resolve(response);
      }

      var error = function(error, status, headers, config){
        // on error then set current user to default value.
        self.resetCookie();
        deferred.reject(error);
      }

      var language = $localstorage.get('language', $translate.use());
      var platform = $localstorage.get('platform', null);
      var device_id = $localstorage.get('device_id', null);
      console.log(source);
      console.log(language);

      if (platform && device_id) {
        $http.get(Constants.API.baseUrl+'/profiles/current.json', {params: {language: language, platform: platform, device_id: device_id}}).success(success).error(error);
      } else {
        $http.get(Constants.API.baseUrl+'/profiles/current.json', {params: {language: language}}).success(success).error(error);
      }

      return deferred.promise;
    },

    // update currentUser's data
    updateUser: function(user,options){
      var self = this;
      var opts = {remove:false, set:false}

      angular.extend(opts,options);
      angular.extend(self.currentUser.info,user);

      if(opts.remove === true){
        window.localStorage.removeItem('app_user');
      }

      if(opts.set === true){
        window.localStorage.setItem('app_user',JSON.stringify(self.currentUser.info));
      }

    },

    isLoggedIn: function(){
      var self = this;
      var output = false;

      if ($rootScope.currentUser) {
        output = true;
      }

      return output;
    },

    //
    verifyRemoteLogin: function(){
      var self = this;

      var deferred = $q.defer();

      var success = function(response, status, headers, config) {
        if(Constants.DEBUGMODE){
          console.log("AuthService.verifyRemoteLogin success callback function ");
          console.log(response);
        }
        var user = response;
        self.updateUser(user, {set: true});
        $rootScope.currentUser = user;
        deferred.resolve(self.currentUser);
      }

      var error = function(error, status, headers, config){
        if(Constants.DEBUGMODE){
          console.log("AuthService.verifyRemoteLogin error callback function ");
          console.log(error);
        }

        deferred.reject(error);
      }

      $http.get(Constants.API.baseUrl+'/profiles/current', {}).success(success).error(error);

      return deferred.promise;
    },

    // registering a user
    register: function(userData){
      var self = this;
      var deferred = $q.defer();

      var language = $localstorage.get('language', $translate.use());

      // success callback
      var success = function(response, status, headers, config){
        if(Constants.DEBUGMODE){
          console.log("AuthService.register success callback");
          console.log(response);
        }

        deferred.resolve(response);
      }

      // error callback
      var error = function(error, status, headers, config){
        if(Constants.DEBUGMODE){
          console.log("AuthService.error callback function ");
          console.log(error);
        }

        deferred.reject(error);
      }

      $http.post(Constants.API.baseUrl+'/users', {user: userData, language: language}).success(success).error(error);

      return deferred.promise;
    },

    // Signing a user in
    login: function(userData,provider){
      var self = this;

      var deferred = $q.defer();

      var success = function(response, status, headers, config){

        var user = response;
        $rootScope.currentUser = user;

        //set token on success
        //$http.defaults.headers.common['X-Access-Token'] = token;
        //window.localStorage.setItem('sessionToken',token);

        deferred.resolve(user);
      }

      var error = function(error, status, headers, config){
        if(Constants.DEBUGMODE){
          console.log("AuthService.error callback function ");
          console.log(error);
        }

        deferred.reject(error);
      }

      // if the provider is a social provider we use inAppbrowser to launch a new window
      // which will redirect the user to the provider's login page
      if(provider === 'instagram' || provider === 'twitter' || provider === 'facebook' || provider === 'tqq') {
        var browserWindow = window.open(Constants.API.baseUrl+'/users/auth/' + provider, "_blank", "closebuttoncaption=Done,location=no");

        // this function is called everytime a new page finishes to load.
        var _checkLogin = function() {
          //var loop = setInterval(function() {
            console.log('checking url....');
            browserWindow.executeScript({code: "document.URL" },
              function(url) {
                var _url = url.toString();
                console.log("_url: ", _url);
                if ((_url.indexOf(Constants.API.baseUrl+"/users/auth/" + provider + "/callback?") > -1) || (_url.indexOf(Constants.API.baseUrl+"/#_=_") > -1) || (_url.indexOf(Constants.API.baseUrl+"/profiles/home") > -1)) {
                  //clearInterval(loop);
                  browserWindow.removeEventListener("loadstop", _checkLogin);
                  browserWindow.removeEventListener("exit", _browserOnClose);
                  browserWindow.executeScript({code: "document.body.innerHTML" },function(response){
                    console.log('Closing browserWindow');
                    //clearInterval(loop);
                    browserWindow.close();
                    _browserOnClose({url: url, response: response});
                  });
                }
              }
            );
          //}, 2000);
        }
        browserWindow.addEventListener( "loadstop", _checkLogin);

        // function called when the browser is closed
        var _browserOnClose = function(output){
          // stripping the html tags out the html response, we only want the json output

          //var _response = JSON.parse(output.response.toString().replace(/(<([^>]+)>)/ig,""));

          // if(Constants.DEBUGMODE){
          //   console.log("==========URL==============");
          //   console.log(output.url.toString());

          //   console.log("==========RESPONSE==============");
          //   console.log(output.response.toString());

          //   console.log(_response);
          // }

          // // if there's a token and a user in the json output everything is fine
          // if(_response.payload.token &&  _response.payload.user){
          //   success(_response);
          // }else{
          //   // if not there was an error
          //   error(_response);
          // }
          self.ping().then(success, error);
        }

        browserWindow.addEventListener( "exit", _browserOnClose);

      }else{
        //default login
        $http.post(Constants.API.baseUrl+'/users/sign_in', {user: userData}).success(success).error(error);
      }

      return deferred.promise;
    },

    // function to sign a user out
    logout: function(){
      var self = this;
      var deferred = $q.defer();

      var success = function(response, status, headers, config){
        self.resetCookie();
        deferred.resolve(response);
      }

      var error = function(error, status, headers, config){
        if(error.status === 400){
          self.resetCookie();
        }
        deferred.reject(error);
      }

      $http.get(Constants.API.baseUrl+'/logout').success(success).error(error);

      return deferred.promise;
    },

    // function to reset data saved till now
    resetCookie: function(){
      var self = this;

      window.localStorage.removeItem('sessionToken');
      $http.defaults.headers.common['X-Access-Token'] = "";

      //self.updateUser({profileBg:"", avatar: "", objectId : "", username: "", name : "", bio:"",location:"",website:""},{remove:true})
    }

  }

  return auth;

}]);

